package GraphEdit::Menufactory ;

use strict ;
use warnings ;
use Carp ;
use Exporter ;

our @ISA       = ('Exporter') ;
our @EXPORT_OK = qw( create_menu create_menu_entry ) ;



# we need menu_names and menus separately: menus is a hash, menu_names is
# an array and therefore keeps order.
sub create_menu {
  my %pars = @_ ;
  my $mw         = $pars{-mw} ;
  #my $menu_names = $pars{-menu_names} ;
  my $menus      = $pars{-menus} ;

  my %menu_widgets;
  my $menu_bar = $mw->Menu( -relief => 'groove' );
  $mw->configure( -menu => $menu_bar );

  while( my $menu_label = shift @$menus ) {

    my $menu_contents = shift @$menus ;
    my $mn = $menu_bar->Menu( -relief => 'groove' );

    while ( my $label = shift @$menu_contents ) {

      my $props = shift @$menu_contents ;
      my $link  = "$menu_label/$label";

      create_menu_entry(
        %pars,
        -props  => $props,
        -menu   => $mn,
        -label  => $label,
        -link   => $link,
      );
    }

    $menu_bar->add('separator') if ( $menu_label eq 'Help' );
    $menu_bar->add(
      'cascade',
      -menu      => $mn,
      -label     => $menu_label,
      -underline => 0
    );

    $menu_widgets{$menu_label} = $mn;
    #$pars{ -ddplot }{menus}{$menu_label} = $mn;
  }

  return \%menu_widgets ;
}

# creates a single menu item like File/Open
sub create_menu_entry {
  my %pars = @_;

  my $props = $pars{-props};
  my $label = $pars{-label};
  my $menu  = $pars{-menu};
  my $link  = $pars{ -link };
  my $mw    = $pars{-mw};

  # @options are passed to the menu command
  my @options = ( -label => $label, -accelerator => $props->[1] );
  my $xxx;
  push @options, @{ $props->[4] } if ( $props->[4] );

  my $comm = 'command';    # determines the type of an entry
  if ( $props->[2] ) { $comm = $props->[2]; }

  # no binding yet for the menu entry: create a disabled menu item
  # we will also not store it in the config hash
  unless ( $props->[0] ) {
    $xxx = $menu->$comm( @options, -state => 'disabled', );
    return $xxx;
  }

  push @options, ( -command => [ \&{ $props->[0] }, %pars, -item => "$label" ] );
  push @options, ( -variable => $props->[3] ) if ( $props->[3] );

  $xxx = $menu->$comm(
    -label => $label,
    @options,
  );

  # store the menu item widget, so we can access it if necessary
  unless ( $comm eq 'separator' ) {
    $pars{-config}{-menus}{$link} = $xxx;
  }

  # create a shortcut in the MainWindow
  #my $shortcut = lc( $props->[1] );
  my $shortcut  = $props->[1] || '' ;
  $shortcut =~ s/-/minus/;
  $shortcut =~ s/\+/plus/;
  $shortcut =~ s/ctrl./Control-/i;
  $shortcut =~ s/shift./Shift-/i;

  if ($shortcut) {
    #print "shortcut=$shortcut\n" ;
    $mw->bind(
      "<$shortcut>" => sub {
        &{ $props->[0] }( %pars, -item => "$label" );
      }
    );
  }

  return $xxx;
}

1 ;
